//+--------------------------------------------------------------------------+
//|                                                                          |
//|                                BassBox                                   |
//|                                                                          |
//|                       GDImage Multimedia Player                          |
//|                                                                          |
//|        Bass.dll, AudioGenie3.dll, WinLIFT64.dll, GDImage64.dll           |
//|                                                                          |
//+--------------------------------------------------------------------------+
//|                                                                          |
//|                         Author Patrice TERRIER                           |
//|                            copyright(c) 2014                             |
//|                           www.zapsolution.com                            |
//|                        pterrier@zapsolution.com                          |
//|                                                                          |
//+--------------------------------------------------------------------------+
//|                  Project started on : 01-20-2014 (MM-DD-YYYY)            |
//|                        Last revised : 02-14-2014 (MM-DD-YYYY)            |
//+--------------------------------------------------------------------------+

#include <windows.h>
#include <WinBase.h>
#include <Shlwapi.h>
#include <Dwmapi.h>
#include <time.h>
#include <locale.h>

#include "Tools.h"
#include "Bass.h"

#pragma comment(lib, "..\\GDImage64.lib")
#pragma comment(lib, "..\\WinLIFT64.lib")
#pragma comment(lib, "..\\Bass64.lib")

// Enable tooltip
#pragma comment(linker,"\"/manifestdependency:type='win32' \
name='Microsoft.Windows.Common-Controls' version='6.0.0.0' \
processorArchitecture='*' publicKeyToken='6595b64144ccf1df' language='*'\"")

const wchar_t*   $BASSMUSICEXT = L".mo3.it.xm.s3m.mtm.mod.umx.";
const wchar_t*   $BASSAUDIOEXT = L".mp3.ogg.wav.mo3.it.xm.s3m.mtm.mod.umx.";
const wchar_t*   $BB64BIN      = L"bb64.bin";
const wchar_t*   $BB64FAV      = L"bb64.fav";
const wchar_t*   $BBP          = L"BBP.exe";

#define STREAM  0
#define MUSIC   1

#define ClientW        254
#define ClientH        449

#define IDC_GDIMAGE   -1
#define IDC_BACKLIST  -2
#define IDC_VOLUME    -3
#define IDC_CAPTION   -4
#define IDC_PLAYLIST  -5
#define IDC_DIRLIST   -6
#define IDC_BBPLIST   -7
#define IDC_FAVORITE  -8

#define IDC_EQU01      -20
#define IDC_EQU02      -19
#define IDC_EQU03      -18
#define IDC_EQU04      -17
#define IDC_EQU05      -16
#define IDC_EQU06      -15
#define IDC_EQU07      -14
#define IDC_EQU08      -13
#define IDC_EQU09      -12
#define IDC_EQU10      -11
#define IDC_EQU_LAST    IDC_EQU10
#define IDC_EQU_FIRST   IDC_EQU01
#define IDC_EQU_OFFON  -21
#define IDC_EQU_RESET  -9

#define IDC_MENU       -39
#define IDC_FAV_ADD    -38
#define IDC_FAV_REMOVE -37
#define IDC_FAV_PLAY   -36
#define IDC_EQUALIZER  -35
#define IDC_LOOPMODE   -34
#define IDC_PLUGIN     -33
#define IDC_PARAM      -32
#define IDC_ABOUT      -31
#define IDC_MENUTITLE  -30

#define BTN_NOTE3      0
#define BTN_BACK       1
#define BTN_MENU       2
#define BTN_MAIN       3

#define ID_MENUBACK1   -1
#define ID_MENUBACK2   -2
#define ID_MENULINE    -3

#define ID_WOOFER      1 // Keep iy at 1; for ShowOscillo
#define ID_APIC        2
#define ID_TIMELINE    3
#define ID_FLARE       4
#define ID_ALBUM       -5
#define ID_TITLE       -6
#define ID_ARTIST      -7
#define ID_GENRE       -8
#define ID_OSCILLO     9 // Note: ID_OSCILLO + (n < 100) is reserved for the oscillo view meter.

#define ID_LOOP       -100
#define ID_TIME       -101
#define ID_DATE       -102

#define ID_CAPTION     201
#define ID_LED         202
#define ID_ANICON      203

#define ID_PLAYBUTTON  1000
#define ID_PLAYON      1001
#define ID_PLAYOFF     1002

#define ID_ARROWBUTTON 2000
#define ID_LARROW      2001 // Left arrow
#define ID_RARROW      2002 // Right arrow

#define ID_BBICON      3000
#define ID_COPYRIGHT   3001

#define OSCILLO_DELAY  30
#define MAIN_TIMER    -1

#define BACKWARD       FALSE
#define FORWARD        TRUE

#define MENU_HIDE      1
#define MENU_SHOW      2

struct SETBIN {
    BYTE equoffon;    // Use Equalizer status (Off/On).
    BYTE equfreq[10]; // Hz percentage for the 10-band equalizer in (the range 0-100).
    BYTE loopmode;    // Playing mode.
    BYTE topmost;     // Topmost mode.
    DWORD pluginsize; // Size of the last plugin used
    BYTE useplugin;   // Shall we use plugin
    DWORD backsize;   // Size of the last background used
};

struct PROP {
    HWND      hwnd;
    LONG_PTR  img;
    HDC       hdc;
    HBITMAP   hbitmap;
    HWND      gdimage;
    HBITMAP   hpeak;
    RECT      r;
    long      oscillomax;
    HWND      hlistback;
    HWND      hlistplugin;
    HINSTANCE instance;
    HWND      caption;
    HWND      hplaylist;
    HWND      hdirlist;
    HWND      menu;
    long      menudo;
    BOOL      menuvisible;
    HWND      hzwp;
    HWND      hfavorite;

    // Audio section
    WCHAR     audiofile[MAX_PATH];
    HSTREAM   channel;       // The playing audio channel
    WORD      lpeak;         // The left audio channel peak value (while in play mode).
    WORD      rpeak;         // The right audio channel peak value (while in play mode).
    LONG_PTR  fftdata;       // DWORD pointer to the FFT() AS SINGLE array.
    LONG_PTR  fftsize;       // Size of the FFT array.
    LONG_PTR  wimdata;       // DWORD pointer to the wave MM_WIM_DATA.
    QWORD     medialength;   // Media length.
    QWORD     mediapos;      // Media pos.
    long      mediatype;     // MUSIC, STREAM, etc.
    long      initdone;      // BassInit flag.
    BOOL      playmode;      // BACKWARD or FORWARD.
    HFX       fx[10];        // FX handle
};

struct BBP {
    long  gdimageid;
    char  plugin[MAX_PATH];
    DWORD nlevel;
    float getdata[256];
    DWORD medialength;
    DWORD mediapos;
    float wimdata[256];
};

PROP gP;
SETBIN gBin;
COPYDATASTRUCT gCDS; BBP gBBP;

WCHAR* BASSBOX() { return L"BassBox"; }

// We use the hidden WinLIFT ID_REMOVEFOCUS button, if available.
void RemoveFocus () {
    HWND hCtrl = GetDlgItem(gP.hwnd, -32756);
    if (IsWindow(hCtrl)) {
        PostMessage (hCtrl, WM_LBUTTONDOWN , 0, 0);
        PostMessage (hCtrl, WM_LBUTTONUP , 0, 0);
    }
}

const void* BassChannelGetData(IN HSTREAM nChannel) {
    static float fft[256];
    memset(&fft[0], 0, sizeof(float)* 256);
    if (nChannel) { long nRet = BASS_ChannelGetData(nChannel, &fft[0], BASS_DATA_FFT512); }
    return &fft[0];
}

long BassFree() {
    long nRet = 0;
    if (gP.initdone) { gP.initdone = 0; nRet = BASS_Free(); }
    return nRet;
}

void HideOscillo() {
    long x = 0, y = 0, w = 0, h = 0;
    for (long nID = 1; nID <= gP.oscillomax; nID++) {
        ZD_SetObjectVisibility(nID, ZS_HIDDEN);
    }
    ZD_SetObjectVisibility(ID_PLAYON, ZS_HIDDEN);
    ZD_SetObjectVisibility(ID_PLAYOFF, ZS_HIDDEN);

    ZD_SetObjectVisibility(ID_LARROW, ZS_HIDDEN);
    ZD_SetObjectVisibility(ID_RARROW, ZS_HIDDEN);
    if (ZD_IsObjectVisible(ID_LOOP)) { ZD_SetObjectVisibility(ID_LOOP, ZS_HIDDEN); }

    ZD_SetObjectVisibility(ID_ALBUM, ZS_HIDDEN); ZD_SetObjectTextEx(ID_ALBUM, L"", FALSE);
    ZD_SetObjectVisibility(ID_TITLE, ZS_HIDDEN); ZD_SetObjectTextEx(ID_TITLE, L"", FALSE);
    ZD_SetObjectVisibility(ID_ARTIST, ZS_HIDDEN); ZD_SetObjectTextEx(ID_ARTIST, L"", FALSE);
    ZD_SetObjectVisibility(ID_GENRE, ZS_HIDDEN); ZD_SetObjectTextEx(ID_GENRE, L"", TRUE);
}

void ShowOscillo() {
    long x = 0, y = 0, w = 0, h = 0, n = 0;
    if (gP.oscillomax == 0) { gP.oscillomax = ID_OSCILLO; }
//    if (gP.mediatype == STREAM) { n = 2; } else { n = 3; }
    if (gBin.useplugin) {  n = 3; } else { n = 2; }
    for (long nID = n; nID <= gP.oscillomax; nID++) {
        ZD_SetObjectVisibility(nID, ZS_VISIBLE);
    }
    ZD_SetObjectFrameToUse(ID_PLAYON, 1, FALSE); ZD_SetObjectVisibility(ID_PLAYON, ZS_HIDDEN);
    ZD_SetObjectFrameToUse(ID_PLAYOFF, 1, FALSE); ZD_SetObjectVisibility(ID_PLAYOFF, ZS_VISIBLE);

    ZD_SetObjectVisibility(ID_ALBUM, ZS_VISIBLE);
    ZD_SetObjectVisibility(ID_TITLE, ZS_VISIBLE);
    ZD_SetObjectVisibility(ID_ARTIST, ZS_VISIBLE);
    ZD_SetObjectVisibility(ID_GENRE, ZS_VISIBLE);
    if (ListCount(gP.hplaylist) > 1) { // Show them only if there is more than one song to play.
        ZD_SetObjectVisibility(ID_LARROW, ZS_VISIBLE);
        ZD_SetObjectVisibility(ID_RARROW, ZS_VISIBLE);
    }
    if (gBin.loopmode) { ZD_SetObjectVisibility(ID_LOOP, ZS_VISIBLE); }
}

long BassCloseChannel() {
    long nRet = 0;
    if (gP.channel) {
        if (gP.mediatype == MUSIC) {
            nRet = BASS_MusicFree(gP.channel);
        } else {
            nRet = BASS_StreamFree(gP.channel);
        }
        gP.channel = 0; gP.mediatype = 0; gP.mediapos = 0;
    }
    HideOscillo();
    return nRet;
}

long BassInit(IN HWND hWnd) {
    if (gP.initdone == 0) { gP.initdone = BASS_Init(1, 44100, 0, hWnd, 0); }
    return gP.initdone;
}

void BBP_Destroy() {
    if (gP.hzwp) {
         gCDS.dwData = 3; // DestroyWindow
         gCDS.cbData = sizeof(gBBP);
         gCDS.lpData = &gBBP;
         SendMessage(gP.hzwp, WM_COPYDATA, (WPARAM) gP.hwnd, (LPARAM) &gCDS);
    }
}

void BBP_Hide() {
    if (gP.hzwp) {
        ShowWindow(gP.hzwp, SW_HIDE);
        ZI_SetFromFile(gP.gdimage, ListGetText(gP.hlistback, ListGetCursel(gP.hlistback)));
    }
}

void BBP_SizeMove() {
    // Use this to move the window in case of AERO shake
    // because setting the new location with SetWindowPos
    // may fail!
    if (gP.hzwp) {
        RECT lpw; GetWindowRect(gP.gdimage, &lpw);
        int x = lpw.left, y = lpw.top, w = lpw.right - x, h = lpw.bottom - y;
        SetWindowPos(gP.hzwp, GetWindow(gP.hwnd, GW_HWNDNEXT), 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE | SWP_NOOWNERZORDER | SWP_NOACTIVATE | SWP_ASYNCWINDOWPOS);
        MoveWindow(gP.hzwp, x, y, w, h, 0);
    }
}

BOOL IsMenuChecked(IN long nID, OUT HWND &hCtrl) {
    BOOL bRet = FALSE;
    hCtrl = 0;
    if ((abs(nID) > abs(IDC_MENUTITLE)) && (abs(nID) < abs(IDC_MENU))) {
       WCHAR szLabel[1] = { 0 };
       hCtrl = GetDlgItem(GetDlgItem(gP.menu, nID), nID);
       GetWindowText(hCtrl, szLabel, sizeof(szLabel));
       if (szLabel[0] == 97) { bRet = TRUE ; }
    }
    return bRet;
}

void BBP_Show() {
    // Use this to move the window in case of AERO shake
    // because setting the new location with SetWindowPos
    // may fail!
    if (gP.hzwp) {

        HWND hCtrl = 0;
        if (IsMenuChecked(IDC_PLUGIN, hCtrl)) {
            WCHAR szFile[MAX_PATH] = { 0 };
            PathCombine(szFile, EXEresource(), L"plugin.png"); // Plugin transparent background
            if (FileExist(szFile)) { ZI_SetFromFile(gP.gdimage, szFile); }
        }

        RECT lpw; GetWindowRect(gP.gdimage, &lpw);
        long x = lpw.left, y = -2000, w = lpw.right - x, h = lpw.bottom - lpw.top;
        SetWindowPos(gP.hzwp, GetWindow(gP.hwnd, 2), 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE | SWP_NOOWNERZORDER | SWP_NOACTIVATE);
        MoveWindow(gP.hzwp, x, y, w, h, 0);
        BBP_SizeMove();
        ShowWindow(gP.hzwp, SW_SHOW);
    }
}

void BBP_UsePlugin(IN WCHAR* szPlugin) {
    if (gP.hzwp) {
        memset(gBBP.plugin, 0, MAX_PATH);
        WideCharToMultiByte(CP_ACP, 0, szPlugin, -1, gBBP.plugin, MAX_PATH, NULL, NULL);
        gCDS.dwData = 2; // Plugin
        gCDS.cbData = sizeof(gBBP);
        gCDS.lpData = &gBBP;
        SendMessage(gP.hzwp, WM_COPYDATA, (WPARAM) gP.hwnd, (LPARAM) &gCDS);
    }
}

void BBP_Handle() {
    if (gP.hzwp == 0) {
        WCHAR szFile[MAX_PATH] = { 0 };
        PathCombine(szFile, EXEpath(), $BBP);
        HINSTANCE nRet = ShellExecute(0, L"open", szFile, L"", EXEpath(), SW_SHOWNOACTIVATE);
        Sleep(400);

        char zClass[] = "ZWALLPAPER";
        gP.hzwp = FindWindowA(zClass, zClass);
        if (gP.hzwp) {
            BBP_SizeMove();

            gBBP.gdimageid = IDC_GDIMAGE;
            gCDS.dwData = 1; // Parent
            gCDS.cbData = sizeof(gBBP);
            gCDS.lpData = &gBBP;
            SendMessage(gP.hzwp, WM_COPYDATA, (WPARAM) gP.hwnd, (LPARAM) &gCDS);

            BBP_UsePlugin(ListGetText(gP.hlistplugin, ListGetCursel(gP.hlistplugin)));
        }
    }
}

void CALLBACK BassStreamEllapsed(IN HSYNC handle, IN DWORD channel, IN DWORD data, void *user) {
    long nID = 0, nItem = 0;
    BassCloseChannel();
    long nCount = ListCount(gP.hplaylist);
    if (nCount > 1) {
        nItem = ListGetCursel(gP.hplaylist);
        if (gP.playmode) { 
            nID = ID_RARROW;
            if ((!gBin.loopmode) && (nItem == 1)) { nID = 0; } // Do not loop.
        } else {
            nID = ID_LARROW;
            if ((!gBin.loopmode) && (nItem == nCount)) { nID = 0; } // Do not loop.
        }
        if (nID) { PostMessage(gP.hwnd, WM_COMMAND, MAKLNG(nID, 1), (LPARAM) gP.gdimage); }
    } else {
        if (gBin.loopmode) { 
            PostMessage(gP.hwnd, WM_COMMAND, MAKLNG(ID_RARROW, 1), (LPARAM) gP.gdimage);
        } else {
            BBP_Hide();
        }
    }
}

void BassSetChannelVolume(IN long nVolume) {
    if (gP.channel) {
        skSetToolTipText(GetDlgItem(gP.hwnd, IDC_VOLUME), STRL(nVolume));
        BASS_ChannelSetAttribute(gP.channel, BASS_ATTRIB_VOL, (float) nVolume / 100.0f); 
    }
}

BOOL IsEqualizerOn() {
    BOOL bRet = FALSE;
    if (ZD_GetObjectFrameToUse(IDC_EQU_OFFON) == 2) { bRet = TRUE; }
    return bRet;
}

void BassUpdateEQ(IN long IDC) {
    BASS_DX8_PARAMEQ p;
    long nItem = IDC - IDC_EQU_FIRST;
    if ((nItem > -1) && (nItem < 10)) {
        gBin.equfreq[nItem] = (BYTE) skGetTrackValue(GetDlgItem(gP.menu, IDC));
        if (gP.channel) {
            if (IsEqualizerOn()) {
               gBin.equoffon = 1; // Equalize is On.
               if (gP.fx[nItem] == 0) {
                   gP.fx[nItem] = BASS_ChannelSetFX(gP.channel, BASS_FX_DX8_PARAMEQ, 0);
                   p.fGain = 0.0f;
                   p.fBandwidth = 18.0f;
                   switch (nItem) {
                   case 0: p.fCenter = 80.0f; break;
                   case 1: p.fCenter = 170.0f; break;
                   case 2: p.fCenter = 310.0f; break;
                   case 3: p.fCenter = 600.0f; break;
                   case 4: p.fCenter = 1000.0f; break;
                   case 5: p.fCenter = 3000.0f; break;
                   case 6: p.fCenter = 6000.0f; break;
                   case 7: p.fCenter = 10000.0f; break;
                   case 8: p.fCenter = 12000.0f; break;
                   case 9: p.fCenter = 14000.0f; break;
                   }
                   BASS_FXSetParameters(gP.fx[nItem], &p);
               }
               BASS_FXGetParameters(gP.fx[nItem], &p);
               p.fGain = (gBin.equfreq[nItem] / 3.3333f) - 15.0f;
               BASS_FXSetParameters(gP.fx[nItem], &p);
            } else {
               gBin.equoffon = 0; // Equalizer is Off.
               if (gP.fx[nItem]) { BASS_ChannelRemoveFX(gP.channel, gP.fx[nItem]); gP.fx[nItem] = 0; }
            }
        }
    }
}

void SetupEqualizer() {
    for (long K = IDC_EQU_FIRST; K <= IDC_EQU_LAST; K++) {
         BassUpdateEQ(K);
    }
}

void CheckMenu(IN long wID) {
    HWND hCtrl = GetDlgItem(GetDlgItem(gP.menu, wID), wID);
    if (IsWindow(hCtrl)) { SetWindowText(hCtrl, L"a"); }
}

void UnCheckMenu(IN long wID) {
    HWND hCtrl = GetDlgItem(GetDlgItem(gP.menu, wID), wID);
    if (IsWindow(hCtrl)) { SetWindowText(hCtrl, L" "); }
}

long BassChannelPlay(IN HWND hWnd) {
    long nRet = 0, nItem = 0;

    WCHAR drive[_MAX_DRIVE], dir[_MAX_DIR], fname[_MAX_FNAME], ext[_MAX_EXT];

    BassCloseChannel();

    if (BassInit(hWnd)) {
        _wsplitpath_s(gP.audiofile, drive, dir, fname, ext);
        CharLower(ext);
        wcsncat_s(ext, L".", 1);
        if (wcsstr($BASSMUSICEXT, ext)) {
            gP.mediatype = MUSIC;
            gP.channel = BASS_MusicLoad(0, gP.audiofile, 0, 0, BASS_STREAM_PRESCAN | BASS_STREAM_AUTOFREE, 0);
        }
        else {
            gP.mediatype = STREAM;
            gP.channel = BASS_StreamCreateFile(0, gP.audiofile, 0, 0, BASS_STREAM_AUTOFREE);
        }

        if (gP.channel) {
            gP.medialength = BASS_ChannelGetLength(gP.channel, 0);

            BYTE *id3tag = (BYTE*)BASS_ChannelGetTags(gP.channel, BASS_TAG_ID3V2);
            if (id3tag) {
                DWORD id3size = 0;
                for (int a = 0; a<4; a++) { id3size = (id3size << 7) | (id3tag[6 + a] & 0x7f); }
                if (id3size) {
                    id3size += 10; // add the size of the ID3v2 header
                }
            }

            long AUDIO_TYPE = AUDIOAnalyzeFileW(gP.audiofile);
            if (AUDIO_TYPE) {
                long nSize = 500000;
                BYTE* buffer = new BYTE[nSize];
                memset(buffer, 0, nSize);
                WCHAR szFile[MAX_PATH] = {0};
                long w = 192, h = 192;

                long nTag = 0;
                // Detect Album cover image.
                if (AUDIO_TYPE == AUDIO_WMA) {
                    nTag = WMAGetPictureCountW();
                    if (nTag) { nTag = WMAGetPictureArrayW(&buffer[0], nSize, 1); }
                } else {
                    nTag = ID3V2GetFrameCountW(ID3F_APIC);
                    if (nTag) { nTag = ID3V2GetPictureArrayW(&buffer[0], nSize, 1); }
                }

                if (nTag) { // Make sure APIC is not corrupted.
                    ZD_SetObjectVisibility(ID_WOOFER, ZS_HIDDEN);
                    PathCombine(szFile, TEMPpath(), L"APIC.jpg");
                    HANDLE hFile = 0;
                    if (zFOpen(szFile, 1, 0, hFile) == 0) {
                        DWORD BytesWritten = 0;
                        WriteFile(hFile, &buffer[0], nTag, &BytesWritten, NULL);
                        zFClose(hFile);

                        HBITMAP hBitmap = ResizeWithVariableOpacity(szFile,w, h);

                        if (hBitmap) { 
                            ZD_DrawBitmapToCtrl(gP.gdimage, (254 - w) / 2, (449 - h) / 2, hBitmap, 0xFFFFFFFF, ID_APIC, ZS_VISIBLE);
                            ZD_SetObjectLocked(ID_APIC, TRUE);
                            ZD_SetObjectZorder(ID_APIC, ZD_ORDER_BOTTOM);
                        }

                        ZD_SetObjectTextEx(ID_ALBUM, GetTagAlbum(AUDIO_TYPE), FALSE);
                        ZD_SetObjectTextEx(ID_TITLE, GetTagTitle(AUDIO_TYPE), FALSE);
                        ZD_SetObjectTextEx(ID_ARTIST, GetTagArtist(AUDIO_TYPE), FALSE);
                        ZD_SetObjectTextEx(ID_GENRE, GetTagGenre(AUDIO_TYPE), TRUE);
                    }
                } else if (gP.mediatype == STREAM) {

                    ZD_SetObjectVisibility(ID_WOOFER, ZS_VISIBLE);
                    PathCombine(szFile, EXEresource(), L"sp01.png");
                    HBITMAP hBitmap = ZI_CreateBitmapFromFile(szFile, w, h);
                    if (hBitmap) { 
                        ZD_DrawBitmapToCtrl(gP.gdimage, (254 - w) / 2, (449 - h) / 2, hBitmap, 0xFFFFFFFF, ID_APIC, ZS_VISIBLE);
                        ZD_SetObjectLocked(ID_APIC, TRUE);
                        ZD_SetObjectZorder(ID_APIC, ZD_ORDER_BOTTOM);
                        ZD_SetObjectZorder(ID_WOOFER, ZD_ORDER_BOTTOM);
                    }

                    ZD_SetObjectTextEx(ID_ALBUM, GetTagAlbum(AUDIO_TYPE), FALSE);
                    ZD_SetObjectTextEx(ID_TITLE, GetTagTitle(AUDIO_TYPE), FALSE);
                    ZD_SetObjectTextEx(ID_ARTIST, GetTagArtist(AUDIO_TYPE), FALSE);
                    ZD_SetObjectTextEx(ID_GENRE, GetTagGenre(AUDIO_TYPE), TRUE);
                }
                delete[] buffer;

            }

            BBP_Show();
            ShowOscillo();
            ZI_UpdateWindow(gP.gdimage, FALSE);

            BASS_ChannelSetSync(gP.channel, BASS_SYNC_END | BASS_SYNC_ONETIME, 0, &BassStreamEllapsed, 0);
            SetupEqualizer();
            BassSetChannelVolume((long) SendMessage(GetDlgItem(gP.hwnd, IDC_VOLUME), TBM_GETPOS, 0, 0));
            nRet = BASS_ChannelPlay(gP.channel, 0);
            nItem = (long) ListFindString(gP.hfavorite, gP.audiofile);
            if (nItem > 0) {
                CheckMenu(IDC_FAV_REMOVE);
                CheckMenu(IDC_FAV_ADD);
            } else {
                UnCheckMenu(IDC_FAV_REMOVE);
                UnCheckMenu(IDC_FAV_ADD);
            }

        }
        else {
            BassCloseChannel();
        }
    }
    return nRet;
}

long BassStreamFree() {
    long nRet = 0;
    if (gP.channel) {
        if (gP.mediatype == MUSIC) {
            nRet = BASS_MusicFree(gP.channel);
        }
        else {
            nRet = BASS_StreamFree(gP.channel);
        }
        gP.channel = 0; gP.mediatype = 0; gP.mediapos = 0;
    }
    HideOscillo();
    return nRet;
}

WORD SolvePeak(IN WORD nValue, IN WORD nTotal) {
    WORD nRet = 0;
    if (nTotal) { nRet = (nValue * 100) / nTotal; }
    return nRet;
}

const void* BassChannelGetWimData(IN HSTREAM nChannel) {
    static float fft[256];
    memset(&fft[0], 0, sizeof(float)* 256);
    if (nChannel) { long nRet = BASS_ChannelGetData(nChannel, &fft[0], 1024); }
    return &fft[0];
}

HBITMAP CreateDIBSection32(IN HDC hDC, IN long nWidth, IN long nHeight) {
    BITMAPINFO bi = { 0 };
    bi.bmiHeader.biSize = sizeof(bi.bmiHeader);
    bi.bmiHeader.biWidth = nWidth;
    bi.bmiHeader.biHeight = -nHeight; // We want to use top left at coordinates 0,0.
    bi.bmiHeader.biPlanes = 1;
    bi.bmiHeader.biBitCount = 32;     // We use 32-bit (alpha channel).
    bi.bmiHeader.biCompression = BI_RGB;
    return CreateDIBSection(hDC, &bi, DIB_RGB_COLORS, NULL, NULL, NULL);
}

void Oscillo(IN DWORD nTick) {
    HWND hCtrl = 0;
    static DWORD nWait;
    if (nWait == 0) { nWait = nTick + OSCILLO_DELAY; }
    if (nTick > nWait) {
        nWait = 0;

        gP.mediapos = BASS_ChannelGetPosition(gP.channel, 0);

        short* pInt = (short*)BassChannelGetWimData(gP.channel);
        short YFactor = 1074;
        long nID = 0, x = 0, y = 0, w = 0, h = 0;
        long nWidth = gP.r.right - gP.r.left;
        long nHeight = (gP.r.bottom - gP.r.top) - 1; // (63)

        if (ZD_GetObjectParent(ID_TIMELINE) == 0) {
            POINTS pXY[2]; 
            pXY[0].x = (short) gP.r.left; pXY[0].y = (short) gP.r.top + 33;
            pXY[1].x = (short) nWidth; pXY[1].y = (short) gP.r.top + 33;
            ZD_DrawPolyLineToCtrl(gP.gdimage, &pXY[0], 2, ZD_ARGB(48, 27, 160, 225), 1, ID_TIMELINE, ZS_VISIBLE, ZD_DRAW_OUTLINE | ZD_DRAW_OPEN, 0);
            ZD_SetObjectLocked(ID_TIMELINE, TRUE);
        }

        nID = ID_OSCILLO - 1; // Make sure the new ID starts at the right range.
        for (x = 0; x < nWidth; x += 8) {
            y = nHeight - ((*pInt + 0x7FFF) / YFactor);
            nID++; gP.oscillomax = nID;
            w = 7; h = y;
            HBITMAP hCrop = ZI_CropBitmap(gP.hpeak, 0, nHeight - y, w, h, 0);
            if (ZD_GetObjectParent(nID)) {
                ZOBJECT zObj = {0};
                ZD_GetObjectProperties(nID, zObj);
                if (zObj.bitmap) { DeleteObject(zObj.bitmap); }
                zObj.x1 = gP.r.left + x;
                zObj.y1 = gP.r.top + nHeight - y;
                zObj.x2 = w;
                zObj.y2 = h;
                zObj.bitmap = hCrop;
                ZD_SetObjectProperties(nID, zObj);
            } else {
                ZD_DrawBitmapToCtrl(gP.gdimage, gP.r.left + x, gP.r.top + y, hCrop, ZD_ColorARGB(255, 0), nID, ZS_VISIBLE);
                ZD_SetObjectLocked(nID, TRUE);
            }
            pInt += 9;
        }

        DWORD nLevel = BASS_ChannelGetLevel(gP.channel); // scan peaks
        float rValue = max(LOINT(nLevel), HIINT(nLevel)) / (float)67200; //44800;
        float Pulse = (rValue / (float)14);
        static float PulseAngle;

        if (gP.hzwp) {
            if (IsMenuChecked(IDC_PLUGIN, hCtrl)) {
                gBBP.nlevel = nLevel;
                //memset(&gBBP.getdata[0], 0, 1024);
                RtlMoveMemory(&gBBP.getdata[0], BassChannelGetData(gP.channel), 1024);//sizeof(float) * 256);
                gBBP.medialength = (DWORD) gP.medialength;
                gBBP.mediapos = (DWORD) gP.mediapos;
                //memset(&gBBP.wimdata[0], 0, 1024);
                RtlMoveMemory(&gBBP.wimdata[0], pInt, 1024);//sizeof(float) * 256);
                gCDS.dwData = 0; // OpenGL
                gCDS.cbData = sizeof(gBBP);
                gCDS.lpData = &gBBP;
                SendMessage(gP.hzwp, WM_COPYDATA, (WPARAM) gP.hwnd, (LPARAM) &gCDS);
            }
        }

        // Draw flare
        long nFrameCount = ZD_GetObjectFrameCount(ID_FLARE);
        ZD_GetObjectBound(ID_FLARE, w, h);
        if (nFrameCount > 1) { w = h; }
        x = (long)((gP.mediapos / (gP.medialength / nWidth)) - (w / 2));
        y = 33 - (h / 2); //(long)(33 - (h / 2));
        ZD_SetObjectXY(ID_FLARE, gP.r.left + x, gP.r.top + y, FALSE);
        //if (w == h) { // Perform flare rotation
        //   PulseAngle += rValue * 5; if (PulseAngle > 360) { PulseAngle = 0; }
        //   ZD_SetObjectAngle(ID_FLARE, (long) PulseAngle, FALSE);
        //}
        long nFrame = ZD_GetObjectFrameToUse(ID_FLARE) + 1; if (nFrame > nFrameCount) { nFrame = 1; }
        ZD_SetObjectFrameToUse(ID_FLARE, nFrame, FALSE);
        ZD_SetObjectZorder(ID_FLARE, ZD_ORDER_TOP);

        // Pulsating APIC or WOOFER
        if (gP.mediatype == STREAM) {
            nID = ID_APIC; if (ZD_IsObjectVisible(ID_WOOFER)) { nID = ID_WOOFER; }
            ZD_SetObjectScale(nID, 1.0f - Pulse);
        }

        ZI_UpdateWindow(gP.gdimage, 0);
    }
}

void Animate() {
    DWORD nTick; long nFrame = 0, x = 0, y = 0, w = 0, h = 0;
    RECT r = { 0 }; POINT p = { 0 };
    static DWORD nSecond = 0, nAniSpeed = 0;

    nTick = GetTickCount();
    if (nSecond == 0) { nSecond = nTick + 1000; }

    if (nTick > nSecond) {
        nSecond = 0; ZD_SetObjectTextEx(ID_TIME, TIME(), TRUE);
    }
    if (gP.channel) {
        Oscillo(nTick); 
    }

    // ICON ANIMATION
    if (nAniSpeed == 0) { nAniSpeed = nTick + 30; }
    if (nTick > nAniSpeed ) {
        nAniSpeed = 0;
        nFrame = ZD_GetObjectFrameToUse(ID_ANICON) + 1; if (nFrame > ZD_GetObjectFrameCount(ID_ANICON)) { nFrame = 1; }
        ZD_SetObjectFrameToUse(ID_ANICON, nFrame, TRUE);
    }
    //// TEXT ANIMATION
    //ZD_GetObjectXY (ID_CAPTION, x, y); if (x > 100) { ZD_SetObjectXY (ID_CAPTION, x - 2, y, FALSE); }

    if (gP.menu) {
        if (gP.menudo) {
            GetWindowRect(gP.menu, &r); p.x = r.left; p.y = r.top;
            w = r.right - r.left; h = r.bottom - r.top;
            ScreenToClient(GetParent(gP.menu), &p);
            GetWindowRect(gP.gdimage, &r);
            if (gP.menudo == MENU_SHOW) {
                if (p.y > (r.bottom - r.top) - h) {
                    gP.menuvisible = TRUE;
                    MoveWindow(gP.menu, p.x, p.y - 8, w, h, FALSE);
                    RedrawWindow(gP.menu, NULL, NULL, RDW_INVALIDATE | RDW_ERASE | RDW_ALLCHILDREN | RDW_UPDATENOW | RDW_ERASENOW);
                } else {
                    MoveWindow(gP.menu, p.x, (r.bottom - r.top) - h, w, h, FALSE);
                    RedrawWindow(gP.menu, NULL, NULL, RDW_INVALIDATE | RDW_ERASE | RDW_ALLCHILDREN | RDW_UPDATENOW | RDW_ERASENOW);
                    gP.menudo = 0;
                }
            } else {
                if (p.y < r.bottom - r.top) {
                    gP.menuvisible = FALSE;
                    MoveWindow(gP.menu, p.x, p.y + 12, w, h, TRUE);
                    ZI_UpdateWindow(gP.gdimage, 0);
                } else {
                    MoveWindow(gP.menu, p.x, r.bottom - r.top, w, h, FALSE);
                    gP.menudo = 0;
                }
            }
        }
    }

}

BOOL CheckPointPos(IN POINT p) {
    BOOL bRet = FALSE;
    if (gP.channel) {
        long h = 0; RECT r = { 0 };
        h = (gP.r.bottom - gP.r.top) / 2;
        SetRect(&r, gP.r.left, gP.r.top + h - 12, gP.r.right, gP.r.top + h + 12);
        bRet = PtInRect(&r, p);
    }
    return bRet;
}

LRESULT CALLBACK CaptionProc(IN HWND hWnd, IN UINT uMsg, IN WPARAM wParam, IN LPARAM lParam) {
    LRESULT nRet = FALSE; // Do not stop the event processing in GDImage

    switch (uMsg) {
    case WM_MOUSEMOVE:
         if (ZI_IsLButtonDown()) { SendMessage(gP.hwnd, WM_NCLBUTTONDOWN, HTCAPTION, NULL); }
         break;
    }
    return nRet;
}

void HideMenuButton () {
    long nMax = (abs(IDC_MENU) - abs(IDC_ABOUT)), nID;
    for (long K = 0; K < nMax; K++) {
         nID = IDC_MENU + K + 1;
         ShowWindow(GetDlgItem(gP.menu, nID), SW_HIDE);
    }
}

void ShowAbout() {
    HideMenuButton ();
    ZD_ShowObject(ID_MENUBACK1, ZS_VISIBLE);
    ZD_ShowObject(ID_MENUBACK2, ZS_VISIBLE);
    ZD_ShowObject(ID_BBICON, ZS_VISIBLE);
    ZD_ShowObject(ID_COPYRIGHT, ZS_VISIBLE);
}

void HideAbout () {
    ZD_ShowObject(ID_BBICON, ZS_HIDDEN);
    ZD_ShowObject(ID_COPYRIGHT, ZS_HIDDEN);
}

void ShowEqualizer () {
    long nID;
    for (long K = IDC_EQU_FIRST; K <= IDC_EQU_LAST; K++) {
         nID = K;
         ShowWindow(GetDlgItem(gP.menu, nID), SW_SHOW);
    }
    ZD_ShowObject(ID_MENUBACK1, ZS_VISIBLE);
    ZD_ShowObject(ID_MENUBACK2, ZS_VISIBLE);
    ZD_ShowObject(ID_MENULINE, ZS_VISIBLE);
    ZD_ShowObject(IDC_EQU_OFFON, ZS_VISIBLE);

    HideMenuButton ();
    ZD_SetObjectTextEx(IDC_MENUTITLE, L"Equalizer", FALSE);
    ShowWindow(GetDlgItem(gP.menu, IDC_EQU_RESET), SW_SHOW);
}

void HideEqualizer () {
    long nID;
    for (long K = IDC_EQU_FIRST; K <= IDC_EQU_LAST; K++) {
         nID = K;
         ShowWindow(GetDlgItem(gP.menu, nID), SW_HIDE);
    }
    ZD_ShowObject(ID_MENUBACK1, ZS_HIDDEN);
    ZD_ShowObject(ID_MENUBACK2, ZS_HIDDEN);
    ZD_ShowObject(ID_MENULINE, ZS_HIDDEN);
    ZD_ShowObject(IDC_EQU_OFFON, ZS_HIDDEN);
    ZD_SetObjectTextEx(IDC_MENUTITLE, L"", FALSE);
    ShowWindow(GetDlgItem(gP.menu, IDC_EQU_RESET), SW_HIDE);
}

void ShowMenuButton () {
    HideAbout();
    HideEqualizer();
    long nMax = (abs(IDC_MENU) - abs(IDC_ABOUT)), nID;
    for (long K = 0; K < nMax; K++) {
         nID = IDC_MENU + K + 1;
         ShowWindow(GetDlgItem(gP.menu, nID), SW_SHOW);
    }
    ZD_SetObjectTextEx(IDC_MENUTITLE, BASSBOX(), FALSE);
    SetCursor(LoadCursor(NULL, IDC_ARROW));
}

void HideMenu () {
    WCHAR szFile[MAX_PATH] = { 0 };
    PathCombine(szFile, EXEresource(), L"blup.wav");
    PlaySound(szFile, NULL, SND_FILENAME | SND_ASYNC);
    gP.menudo = MENU_HIDE;
    gP.menuvisible = FALSE;
}

void skUpdateWindow (IN HWND hWnd, IN long nFlagMode) {
    InvalidateRect(hWnd, NULL, nFlagMode);
    UpdateWindow(hWnd);
}

HDC skOFFscreen(IN HDC hDC, IN long x, IN long y, IN long Action) {
    static HDC UseDC, hDCTemp;
    static HBITMAP hBMTemp;
    static long xx, yy;

    if (Action) {
       // Create OFF screen bitmap to avoid flickering and allow smooth display
       // ---------------------------------------------------------------------
       hDCTemp = CreateCompatibleDC(hDC);
       hBMTemp = CreateCompatibleBitmap(hDC, x, y);
       SelectObject(hDCTemp, hBMTemp);
       xx = x; yy = y;
       UseDC = hDC; }
    else {
       // End of OFF screen drawing, fast BitBlt to the target display Window
       // -------------------------------------------------------------------
       // Draw final result to the target window
       BitBlt(UseDC, x, y, xx, yy, hDCTemp, 0, 0, SRCCOPY);
       // Deallocate system resources
       DeleteObject(hBMTemp);
       DeleteDC(hDCTemp);
       hDCTemp = 0;
    }
    return hDCTemp;
}

void SetEqualizeOn() {
    ZD_SetObjectFrameToUse(IDC_EQU_OFFON, 2, FALSE);
    HWND hCtrl = GetDlgItem(GetDlgItem(gP.menu, IDC_EQUALIZER), IDC_EQUALIZER);
    if (hCtrl) { SetWindowText(hCtrl, L"a"); }
}

void SetEqualizeOff() {
    ZD_SetObjectFrameToUse(IDC_EQU_OFFON, 1, FALSE);
    HWND hCtrl = GetDlgItem(GetDlgItem(gP.menu, IDC_EQUALIZER), IDC_EQUALIZER);
    if (hCtrl) { SetWindowText(hCtrl, L" "); }
}

void DrawOffOnButton (IN long nFrame) {
     long x = 0, y = 0, w = 0, h = 0;
     HDC hDC = 0, hDCTemp = 0;
     // To avoid potential flickering when using RedrawWindow with the trackbars
     ZD_GetObjectXY(IDC_EQU_OFFON, x, y); ZD_GetObjectBound(IDC_EQU_OFFON, w, h);
     hDC = GetDC(gP.menu); hDCTemp = CreateCompatibleDC(hDC);
     SelectObject(hDCTemp, ZD_GetObjectBitmap (IDC_EQU_OFFON));
     BitBlt(hDC, x, y, h, h, hDCTemp, h * nFrame - h, 0, SRCCOPY);
     DeleteDC(hDCTemp);  ReleaseDC(gP.menu, hDC);
}

LRESULT CALLBACK MenuProc(IN HWND hWnd, IN UINT uMsg, IN WPARAM wParam, IN LPARAM lParam) {
    HWND hCtrl = 0;
    LRESULT nRet = FALSE; // Do not stop the event processing in GDImage
    long wID = 0, nObjectID = 0, nFrame = 0, nCount = 0, K = 0, nID = 0, nAdd = 0, nItem = 0;
    WCHAR szFile[MAX_PATH] = { 0 };

    switch (uMsg) {
    case WM_SETCURSOR:
         nRet = 1;
         break;

    case WM_LBUTTONUP:
         nObjectID = ZI_MouseOverObjectID();
         if (nObjectID == IDC_EQU_OFFON) {
             nCount = ZD_GetObjectFrameCount(nObjectID);
             nFrame = ZD_GetObjectFrameToUse(nObjectID) + 1; if (nFrame > nCount) { nFrame = 1; }
             ZD_SetObjectFrameToUse(nObjectID, nFrame, FALSE);
             hCtrl = GetDlgItem(GetDlgItem(gP.menu, IDC_EQUALIZER), IDC_EQUALIZER);
             if ((nFrame == 1) && (hCtrl)) { 
                 SetWindowText(hCtrl, L" "); gBin.equoffon = (BYTE) 0;
             } else { 
                 SetWindowText(hCtrl, L"a"); gBin.equoffon = (BYTE) 1;
             }

             DrawOffOnButton(nFrame);

             SetupEqualizer();

         }
         break;

    case WM_COMMAND:
         RemoveFocus(); // Remove focus from button, to preserve mousehover effect.

         wID = LOINT(wParam);
         if (wID > IDC_EQUALIZER) {
             if (IsMenuChecked(wID, hCtrl)) {
                 UnCheckMenu(wID);
                 if (wID == IDC_LOOPMODE) { gBin.loopmode = (BYTE) 0; }
             } else {
                 CheckMenu(wID);
                 if (wID == IDC_LOOPMODE) { gBin.loopmode = (BYTE) 1; }
             }
         }

         switch (wID) {

         case IDC_EQU_RESET:
              for (K = IDC_EQU_FIRST; K <= IDC_EQU_LAST; K++) {
                  nID = K; skSetTrackValue(GetDlgItem(gP.menu, nID), 50);
                  BassUpdateEQ(nID);
              }
              hCtrl = GetDlgItem(GetDlgItem(gP.menu, IDC_EQUALIZER), IDC_EQUALIZER);
              nFrame = 1;
              if (hCtrl) { SetWindowText(hCtrl, L" "); gBin.equoffon = (BYTE) 0; }
              DrawOffOnButton(nFrame);
              break;

         case IDC_EQU01:
         case IDC_EQU02:
         case IDC_EQU03:
         case IDC_EQU04:
         case IDC_EQU05:
         case IDC_EQU06:
         case IDC_EQU07:
         case IDC_EQU08:
         case IDC_EQU09:
         case IDC_EQU10:
              BassUpdateEQ(wID);
              break;

         case IDC_ABOUT:
              ShowAbout();
              break;

         case IDC_PARAM:
              if (IsMenuChecked(wID, hCtrl)) {
                  SetWindowPos(gP.hwnd, HWND_TOPMOST, 0, 0, 0, 0, SWP_NOSIZE | SWP_NOMOVE | SWP_SHOWWINDOW);
                  gBin.topmost = 1;
              } else {
                  SetWindowPos(gP.hwnd, HWND_NOTOPMOST, 0, 0, 0, 0, SWP_NOSIZE | SWP_NOMOVE | SWP_SHOWWINDOW);
                  gBin.topmost = 0;
              }
              break;

         case IDC_PLUGIN:
              if (IsMenuChecked(wID, hCtrl)) {
                  BBP_Show();
                  gBin.useplugin = TRUE;
              } else {
                  BBP_Hide();
                  gBin.useplugin = FALSE;
              }
              break;

         case IDC_LOOPMODE:
              if ((gP.channel) && (gBin.loopmode)) {
                  if (ZD_IsObjectVisible(ID_LOOP) == 0) { ZD_SetObjectVisibility(ID_LOOP, ZS_VISIBLE); }
              } else {
                  if (ZD_IsObjectVisible(ID_LOOP)) { ZD_SetObjectVisibility(ID_LOOP, ZS_HIDDEN); }
              }
              break;

         case IDC_EQUALIZER:
              ShowEqualizer();
              break;

         case IDC_FAV_PLAY:
              nCount = ListCount(gP.hfavorite);
              if (nCount > 0) {
                  ListDeleteAll(gP.hplaylist);
                  nAdd = 0;
                  for (K = 1; K <= nCount; K++) {
                      if (ListAdd(gP.hplaylist, ListGetText(gP.hfavorite, K)) > -1) { nAdd++; }
                  }
                  if (nAdd) {
                      ListSelectPlus(gP.hplaylist, 1);
                      wcscpy_s(gP.audiofile, ListGetText(gP.hplaylist, ListGetCursel(gP.hplaylist)));
                      BassChannelPlay(hWnd);
                      CheckMenu(wID);
                  }
              }
              break;

         case IDC_FAV_REMOVE:
              if (IsMenuChecked(wID, hCtrl)) {
                  nItem = (long) ListFindString(gP.hfavorite, ListGetText(gP.hplaylist, ListGetCursel(gP.hplaylist)));
                  if (nItem > 0) {
                      ListDeleteItem(gP.hfavorite, nItem);
                      UnCheckMenu(IDC_FAV_ADD);
                      UnCheckMenu(IDC_FAV_REMOVE);
                  }
              }
              break;

         case IDC_FAV_ADD:
              nItem = (long) ListFindString(gP.hfavorite, gP.audiofile);
              if (nItem < 1) {
                  ListAdd(gP.hfavorite, gP.audiofile);
                  CheckMenu(IDC_FAV_ADD);
                  CheckMenu(IDC_FAV_REMOVE);
              }
              break;
         }
         break;
    }
    return nRet;
}

LRESULT CALLBACK GDImageCallBack(IN HWND hWnd, IN UINT uMsg, IN WPARAM wParam, IN LPARAM lParam) {
    HWND hCtrl = 0;
    LRESULT nRet = FALSE; // Do not stop the event processing in GDImage

    WCHAR szLabel[64] = { 0 };
    long nCount = 0, nObjectID = 0, nListItem = 0, x = 0, y = 0, w = 0, h = 0; 
    QWORD nPos = 0; POINT p = { 0 };
    static long nOscilloID;
    static WCHAR wasLabel[64];

    nObjectID = ZI_MouseOverObjectID(); if (nObjectID) { nCount = ZD_GetObjectFrameCount(nObjectID); }
    if (nObjectID > 99) { nOscilloID = 0; }

    switch (uMsg) {
    case WM_LBUTTONDOWN:
    case WM_LBUTTONDBLCLK:
         p.x = LOINT(lParam); p.y = HIINT(lParam);
         if (CheckPointPos(p)) { // We are on the oscillo
             if (gP.channel) {
                 nPos = (gP.medialength / 254) * p.x;
                 BASS_ChannelSetPosition(gP.channel, nPos, 0);
             }
             nOscilloID = nObjectID;
         } else if (nObjectID <= IDC_GDIMAGE) {
             if ((gP.hzwp) && (IsMenuChecked(IDC_PLUGIN, hCtrl))) {
                 nListItem = ListGetCursel(gP.hlistplugin);
                 if (nListItem > 1) {
                     --nListItem;
                     ListSelectPlus(gP.hlistplugin, nListItem);
                 } else {
                     nListItem = ListCount(gP.hlistplugin);
                     ListSelectPlus(gP.hlistplugin, nListItem);
                 }
                 BBP_UsePlugin(ListGetText(gP.hlistplugin, ListGetCursel(gP.hlistplugin)));
             } else {
                 nListItem = ListGetCursel(gP.hlistback);
                 if (nListItem > 1) {
                     --nListItem;
                     ListSelectPlus(gP.hlistback, nListItem);
                 } else {
                     nListItem = ListCount(gP.hlistback);
                     ListSelectPlus(gP.hlistback, nListItem);
                 }
                 ZI_SetFromFile(gP.gdimage, ListGetText(gP.hlistback, ListGetCursel(gP.hlistback)) );
             }
         } else if ((nObjectID > ID_PLAYBUTTON) && (nCount == 5)) {
             if (ZD_GetObjectFrameToUse(nObjectID) != 2) { ZD_SetObjectFrameToUse(nObjectID, 2, TRUE); }
         }

         break;
    case WM_RBUTTONDOWN:
    case WM_RBUTTONDBLCLK:
         p.x = LOINT(lParam); p.y = HIINT(lParam);
         if (CheckPointPos(p)) { // We are on the oscillo
             if (gP.channel) {
                 nPos = (gP.medialength / 254) * p.x;
                 BASS_ChannelSetPosition(gP.channel, nPos, 0);
             }
             nOscilloID = nObjectID;
         } else if (nObjectID <= IDC_GDIMAGE) {
             if ((gP.hzwp) && (IsMenuChecked(IDC_PLUGIN, hCtrl))) {
                 nListItem = ListGetCursel(gP.hlistplugin);
                 if (nListItem < ListCount(gP.hlistplugin)) {
                     ++nListItem;
                     ListSelectPlus(gP.hlistplugin, nListItem);
                 } else {
                     nListItem = 1;
                     ListSelectPlus(gP.hlistplugin, nListItem);
                 }
                 BBP_UsePlugin(ListGetText(gP.hlistplugin, ListGetCursel(gP.hlistplugin)));
             } else {
                 nListItem = ListGetCursel(gP.hlistback);
                 if (nListItem < ListCount(gP.hlistback)) {
                     ++nListItem;
                     ListSelectPlus(gP.hlistback, nListItem); 
                 } else {
                     nListItem = 1;
                     ListSelectPlus(gP.hlistback, nListItem); 
                 }
                 ZI_SetFromFile(gP.gdimage, ListGetText(gP.hlistback, ListGetCursel(gP.hlistback)) );
             }
         }
         break;

    case WM_LBUTTONUP:
         if (nOscilloID > 0) {
             nOscilloID = 0;
         }
         if (nObjectID > ID_ARROWBUTTON) {
             // Fire WM_COMMAND event there
             PostMessage(gP.hwnd, WM_COMMAND, MAKLNG(nObjectID, 1), (LPARAM) hWnd);

         } else if (nObjectID > ID_PLAYBUTTON) {
             ZD_SetObjectFrameToUse(nObjectID, 5, TRUE);
             PostMessage(gP.hwnd, WM_COMMAND, MAKLNG(nObjectID, 1), (LPARAM) hWnd);
         }
         break;

    case WM_MOUSEMOVE:

         // Hide menu if visible.
         if (gP.menuvisible) { HideMenu(); }

         if (ZD_IsObjectVisible(ID_LARROW)) {
             ZD_GetObjectXY(ID_LARROW, x, y);
             if (nObjectID < ID_ARROWBUTTON) {
                 if (x > -1) {
                     ZD_GetObjectBound (ID_LARROW, w, h);
                     ZD_SetObjectXY(ID_LARROW, -w + 3, y, FALSE);
                     ZD_SetObjectXY(ID_RARROW, 254 - 3, y, FALSE);
                 }
             } else { // we are above one of the ARROWBUTTON

                 // REM this out to display tooltip
//                 wcscpy_s(szLabel, ZD_GetObjectImageLabel(nObjectID));
//                 if (_wcsicmp(szLabel, wasLabel) !=0) { ZI_SetToolTipText(hWnd, szLabel); }
//                 wcscpy_s(wasLabel, szLabel);

                 if (x < 0) {
                     ZD_GetObjectBound (ID_LARROW, w, h);
                     ZD_SetObjectXY(ID_LARROW, 0, y, FALSE);
                     ZD_SetObjectXY(ID_RARROW, 254 - w, y, FALSE);
                 }
                 break;
             }
         }

         // REM this out when using tooltip
//         if (wcslen(wasLabel)) { wcscpy_s(wasLabel, L"");  ZI_SetToolTipText(hWnd, wasLabel); }

         p.x = LOINT(lParam); p.y = HIINT(lParam);
         if (nOscilloID) {
             if (ZI_IsLButtonDown()) {
                 if (gP.channel) {
                     if (CheckPointPos(p)) { // We are on the oscillo
                         nPos = (gP.medialength / 254) * p.x;
                         BASS_ChannelSetPosition(gP.channel, nPos, 0);
                     }
                 }
             }
         } else if (nObjectID > ID_PLAYBUTTON) {
             if (ZD_GetObjectFrameToUse(nObjectID) == 1) {
                 ZD_SetObjectFrameToUse(nObjectID, 5, TRUE);
             }
         } else {
             if (ZD_GetObjectFrameCount(ID_PLAYON) == 5) { ZD_SetObjectFrameToUse(ID_PLAYON, 1, FALSE); }
             if (ZD_GetObjectFrameCount(ID_PLAYOFF) == 5) { ZD_SetObjectFrameToUse(ID_PLAYOFF, 1, FALSE); }
         }
         break;
    }
    return nRet;
}

void GDImageEnableEvents(IN HWND hGDImageCtrl) {
    LONG_PTR lpCallBack = (LONG_PTR) GDImageCallBack;
    ZI_EventMessageEx(hGDImageCtrl, lpCallBack, WM_LBUTTONDOWN, TRUE);
    ZI_EventMessageEx(hGDImageCtrl, lpCallBack, WM_LBUTTONUP, TRUE);
    ZI_EventMessageEx(hGDImageCtrl, lpCallBack, WM_LBUTTONDBLCLK, TRUE);
    ZI_EventMessageEx(hGDImageCtrl, lpCallBack, WM_RBUTTONDOWN, TRUE);
    ZI_EventMessageEx(hGDImageCtrl, lpCallBack, WM_RBUTTONDBLCLK, TRUE);
    ZI_EventMessageEx(hGDImageCtrl, lpCallBack, WM_KEYDOWN, TRUE);
    ZI_EventMessageEx(hGDImageCtrl, lpCallBack, WM_KEYUP, TRUE);  
    ZI_EventMessageEx(hGDImageCtrl, lpCallBack, WM_MOUSEMOVE, TRUE);
}

//////////////////////////////////////////////////////////////////////////////////
// Procedures to perform the audio file detection.
long BassCheckExtension(IN WCHAR* zFileName) {
    long nRet = 0;
    WCHAR drive[_MAX_DRIVE], dir[_MAX_DIR], fname[_MAX_FNAME], ext[_MAX_EXT + 1];
    if (zFileName) {
        _wsplitpath_s(zFileName, drive, dir, fname, ext);
        if (wcslen(ext)) {
            wcsncat_s(ext, L".", 1);
            CharLower(ext);
            if (wcsstr($BASSAUDIOEXT, ext)) { nRet = -1; }
        }
    }
    return nRet;
}

void ReadDirs(IN WCHAR* szDir) {
    HANDLE hFind = 0;
    WIN32_FIND_DATA fd = {0};
    WCHAR zPath[MAX_PATH] = { 0 };
    wcscpy_s(zPath, szDir);
    wcsncat_s(zPath, L"*", 1);

    hFind = FindFirstFile(zPath, &fd);
    if (hFind != INVALID_HANDLE_VALUE) {
        do {
            if (fd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) { // If directory bit (16) is set
                if (fd.cFileName[0] != 46) { // No . nor ..
                    PathCombine(zPath, szDir, fd.cFileName);
                    PathAddBackslash(zPath);
                    ListAdd(gP.hdirlist, zPath);
                    // We have a subdir, call function again to find others
                    // Each recursive call gets its own place on the stack
                    ReadDirs(zPath);
                }
            }
        } while (FindNextFile(hFind, &fd) != 0);
        FindClose(hFind);
    }
}

long DetectAudio (IN WCHAR* szDir) {
    HANDLE hFind = 0;
    WIN32_FIND_DATA fd = {0};
    WCHAR zPath[MAX_PATH] = { 0 };
    wcscpy_s(zPath, szDir);
    wcsncat_s(zPath, L"*.*", 3);

    long nCount = 0;
    hFind = FindFirstFile(zPath, &fd);
    if (hFind != INVALID_HANDLE_VALUE) {
        do {
            if (fd.dwFileAttributes & 39) { // NORMAL + READONLY + HIDDEN + SYSTEM + ARCHIV
                PathCombine(zPath, szDir, fd.cFileName);
                if (BassCheckExtension(zPath)) {
                    if (ListAdd(gP.hplaylist, zPath) > -1 ) {
                        nCount++;
                    }
                }
            }
        } while (FindNextFile(hFind, &fd) != 0);
        FindClose(hFind);
    } 
    return nCount;
}

long RecursiveSearch(IN WCHAR* szDir) {
    long nDirCount = 0, nFileCount = 0;
    ListDeleteAll(gP.hdirlist);
    WCHAR szBuffer[MAX_PATH];
    wcscpy_s(szBuffer, szDir); // Prevent the possibility of a buffer overrun.
    PathAddBackslash(szBuffer);
    ListAdd(gP.hdirlist, szBuffer);
    //CursorWait()
    ReadDirs(szBuffer);
    nDirCount = ListCount(gP.hdirlist);
    for (long k = 1; k <= nDirCount; k++) {
        nFileCount += DetectAudio(ListGetText(gP.hdirlist, k));
    }
    //CursorStd()
    return nFileCount;
}

void ProccessCommandline(IN HWND hWnd, IN LPTSTR lpCmdLine) {
    if (wcslen(lpCmdLine)) { 
        LPWSTR *szArglist;
        int nArgs;
        long nCount = 0, k;
        szArglist = CommandLineToArgvW(lpCmdLine, &nArgs);
        if (nArgs > 0) {
            ListDeleteAll(gP.hplaylist);
            for (k = 0; k < nArgs; k++) {
                if (BassCheckExtension(szArglist[k])) {
                    if (ListAdd(gP.hplaylist, szArglist[k]) > -1) {
                        nCount++;
                    }
                } else {
                    nCount += RecursiveSearch(szArglist[k]);
                }
            }
            // Free memory allocated for CommandLineToArgvW arguments.
            LocalFree(szArglist);
            if (nCount) {
                ListSelectPlus(gP.hplaylist, 1);
                wcscpy_s(gP.audiofile, ListGetText(gP.hplaylist, ListGetCursel(gP.hplaylist)));
                BassChannelPlay(hWnd);
                UnCheckMenu(IDC_FAV_PLAY);
            }
        }
    }
}

void SaveFavorite() {
    WCHAR szFile[MAX_PATH] = { 0 };
    BOOL bErrorFlag = FALSE; 
    DWORD dwBytesWritten = 0;
    long nCount = ListCount(gP.hfavorite);
    PathCombine(szFile, TEMPpath(), $BB64FAV);
    if (nCount) {
        HANDLE hFile = CreateFile(szFile, GENERIC_WRITE, 0, NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
        if (hFile != INVALID_HANDLE_VALUE) { 
            for (long K = 1; K <= nCount; K++) {
                 memset(szFile, 0, strSize(szFile));
                 wcscpy_s(szFile, ListGetText(gP.hfavorite, K));
                 wcsncat_s(szFile, L"\n", 1);
                 bErrorFlag = WriteFile(hFile, szFile, (long) wcslen(szFile) * sizeof(WCHAR), &dwBytesWritten, NULL);
            }
            CloseHandle(hFile);
        }
    } else {
        DeleteFile(szFile);
    }
}

//////////////////////////////////////////////////////////////////////////////////

LRESULT CALLBACK MainProc(IN HWND hWnd, IN UINT uMsg, IN WPARAM wParam, IN LPARAM lParam) {
    HDROP hfInfo = { 0 };
    WCHAR szFile[MAX_PATH] = { 0 };
    long K = 0, nDrop = 0, nRet = 0, nLED = 2, nCount = 0, nItem = 0;
    RECT r = { 0 }; POINT p = { 0 };
    COPYDATASTRUCT* cds = { 0 };

    switch (uMsg) {

    case WM_TIMER:
        Animate();
        break;

    case WM_COPYDATA: // Read the data from the command line, while BB64 is already running.
        RtlMoveMemory(&cds, &lParam, sizeof(cds));
        cds = (COPYDATASTRUCT*) lParam;
        if (wParam == 0) { // Command line
            RtlMoveMemory(szFile, cds->lpData, cds->cbData);
            ProccessCommandline(hWnd, szFile);
        }
        break;

    case WM_ACTIVATEAPP:
//    case WM_IME_SETCONTEXT:
        if (wParam) { nLED = 1; }
        ZD_SetObjectFrameToUse(ID_LED, nLED, TRUE);
        break;

    case WM_NCHITTEST:
        return HTCAPTION;

    case WM_VSCROLL:
        nRet = GetDlgCtrlID((HWND) lParam);
        if (nRet == IDC_VOLUME) { BassSetChannelVolume((long) SendMessage((HWND) lParam, TBM_GETPOS, 0, 0)); }
        break;

    case WM_SIZE:
    case WM_MOVE:
    case WM_WINDOWPOSCHANGED:
    case WM_IME_NOTIFY:
         BBP_SizeMove();
         break;

    case WM_COMMAND:
        switch (LOINT(wParam)) {

        case BTN_BACK:
             break;

        case BTN_MENU:
             if (gP.menu) {
                 GetWindowRect(gP.menu, &r); p.x = r.left; p.y = r.top;
                 ScreenToClient(GetParent(gP.menu), &p);
                 GetWindowRect(gP.gdimage, &r);
                 if (p.y < r.bottom - r.top) {
                     HideMenu();
                 } else {
                     ShowMenuButton();
                     gP.menudo = MENU_SHOW;
                     PathCombine(szFile, EXEresource(), L"blup.wav");
                     PlaySound(szFile, NULL, SND_FILENAME | SND_ASYNC);
                 }
             }
             break;

        case BTN_MAIN:
             PostMessage(hWnd, WM_SYSCOMMAND, SC_CLOSE, 0);
             break;

        case ID_PLAYON:
             if (gP.channel) {
                 ZD_SetObjectVisibility(ID_PLAYON, ZS_HIDDEN);
                 BASS_ChannelPlay(gP.channel, FALSE);
                 ZD_SetObjectFrameToUse(ID_PLAYOFF, 1, FALSE);
                 ZD_SetObjectZorder(ID_PLAYOFF, ZD_ORDER_TOP); // Put this button on top of z-order.
                 ZD_SetObjectVisibility(ID_PLAYOFF, ZS_VISIBLE);
             }
             break;

        case ID_PLAYOFF:
             if (gP.channel) {
                 ZD_SetObjectVisibility(ID_PLAYOFF, ZS_HIDDEN);
                 BASS_ChannelPause(gP.channel);
                 ZD_SetObjectFrameToUse(ID_PLAYON, 1, FALSE);
                 ZD_SetObjectZorder(ID_PLAYON, ZD_ORDER_TOP); // Put this button on top of z-order.
                 ZD_SetObjectVisibility(ID_PLAYON, ZS_VISIBLE);
             }
             break;

        case ID_LARROW:
             nItem = ListGetCursel(gP.hplaylist);
             if (nItem > 1) {
                 ListSelectPlus(gP.hplaylist, nItem - 1);
             } else {
                 ListSelectPlus(gP.hplaylist, ListCount(gP.hplaylist));
             }
             wcscpy_s(gP.audiofile, ListGetText(gP.hplaylist, ListGetCursel(gP.hplaylist)));
             gP.playmode = BACKWARD;
             BassChannelPlay(hWnd);
             break;

        case ID_RARROW:
             nItem = ListGetCursel(gP.hplaylist);
             if (nItem < ListCount(gP.hplaylist)) {
                 ListSelectPlus(gP.hplaylist, nItem + 1);
             } else {
                 ListSelectPlus(gP.hplaylist, 1);
             }
             wcscpy_s(gP.audiofile, ListGetText(gP.hplaylist, ListGetCursel(gP.hplaylist)));
             gP.playmode = FORWARD;
             BassChannelPlay(hWnd);
             break;
        }
        break;

    case WM_DROPFILES:
        hfInfo = HDROP(wParam);
        nDrop = DragQueryFileA(hfInfo, 0xFFFFFFFF, NULL, 0);
        nCount = 0;
        if (nDrop > 0) {
            ListDeleteAll(gP.hplaylist);
            for (K = 0; K < nDrop; K++) {
                nRet = DragQueryFile(hfInfo, K, szFile, MAX_PATH);
                CharLower(szFile);
                if (BassCheckExtension(szFile)) {
                    if (ListAdd(gP.hplaylist, szFile) > -1 ) { nCount++; }
                } else {
                    nCount += RecursiveSearch(szFile);
                }
            }
        }
        DragFinish(hfInfo);
        if (nCount) {
            ListSelectPlus(gP.hplaylist, 1);
            wcscpy_s(gP.audiofile, ListGetText(gP.hplaylist, ListGetCursel(gP.hplaylist)));
            BassChannelPlay(hWnd);
            UnCheckMenu(IDC_FAV_PLAY);
        }
        SetForegroundWindow(hWnd);
        break;

    case WM_DESTROY:
        SaveFavorite();
        if (IsWindow(gP.hlistplugin)) { gBin.pluginsize = FileSize(ListGetText(gP.hlistplugin, ListGetCursel(gP.hlistplugin))); }
        if (IsWindow(gP.hlistback)) { gBin.backsize = FileSize(ListGetText(gP.hlistback, ListGetCursel(gP.hlistback))); }
        BBP_Destroy();
        BassCloseChannel();
        if (gP.hbitmap) { DeleteObject(gP.hbitmap); }
        if (gP.hdc) { DeleteDC(gP.hdc); }
        PostQuitMessage(0);
        if (gP.img) { GdipDisposeImage(gP.img); gP.img = 0; }
        return 0;
    }

    return DefWindowProc(hWnd, uMsg, wParam, lParam);
}

void DetectBackground() {
    HANDLE hFind = 0;
    WIN32_FIND_DATA fd = {0};
    WCHAR szSearch[MAX_PATH] = { 0 };
    WCHAR szBackground[MAX_PATH] = { 0 };
    WCHAR BACKresource[MAX_PATH] = { 0 };
    PathCombine(BACKresource, EXEresource(), L"background\\");

    if (gP.hlistback == 0) {
        gP.hlistback = CreateWindowEx(0, L"LISTBOX", NULL, WS_CHILD | LBS_SORT,
                                      0, -1000, 0, 0, gP.hwnd, (HMENU) IDC_BACKLIST, gP.instance, NULL);
    } else {
        ListDeleteAll(gP.hlistback);
    }

    wcscpy_s(szBackground, L"");

    PathCombine(szSearch, BACKresource, L"bk??.png");
    hFind = FindFirstFile(szSearch, &fd);
    if (hFind != INVALID_HANDLE_VALUE) {
        do {
            if (fd.cFileName[0] != 46) {
                switch (fd.dwFileAttributes) {
                case 0:
                case 1:
                case 32:
                case 33:
                case 128:
                     PathCombine(szSearch, BACKresource, fd.cFileName);
                     CharLower(szSearch);
                     if (ListAdd(gP.hlistback, szSearch) > -1) {
                         // if (same size,) then we assume it is our current background...
                         if (fd.nFileSizeLow == gBin.backsize) { wcscpy_s(szBackground, szSearch); }
                     }
                     break;
                }
            }
        }
        while (FindNextFile(hFind, &fd) != 0);
        FindClose(hFind);
    }
    long nSelected = (long) ListFindString(gP.hlistback, szBackground);
    ListSelectPlus(gP.hlistback, nSelected);
    if (nSelected < 0) {
        nSelected = 1;
        gBin.backsize = FileSize(ListGetText(gP.hlistback, nSelected));
    }
    ListSelectPlus(gP.hlistback, nSelected);
    skSkinDisable(gP.hlistback);
}

void DetectPlugin() {
    HANDLE hFind = 0;
    WIN32_FIND_DATA fd = {0};
    WCHAR szSearch[MAX_PATH] = { 0 };
    WCHAR szPlugin[MAX_PATH] = { 0 };
    WCHAR BBPresource[MAX_PATH] = { 0 };
    PathCombine(BBPresource, EXEpath(), L"BBplugin\\");

    if (gP.hlistplugin == 0) {
        gP.hlistplugin = CreateWindowEx(0, L"LISTBOX", NULL, WS_CHILD | LBS_SORT,
                                      0, -1000, 0, 0, gP.hwnd, (HMENU) IDC_BBPLIST, gP.instance, NULL);
        if (gP.hlistplugin == 0) { return; }
    }

    wcscpy_s(szPlugin, L"");

    PathCombine(szSearch, BBPresource, L"BBP_*.dll");
    hFind = FindFirstFile(szSearch, &fd);
    if (hFind != INVALID_HANDLE_VALUE) {
        do {
            if (fd.cFileName[0] != 46) {
                switch (fd.dwFileAttributes) {
                case 0:
                case 1:
                case 32:
                case 33:
                case 128:
                     PathCombine(szSearch, BBPresource, fd.cFileName);
                     CharLower(szSearch);
                     if (ListAdd(gP.hlistplugin, szSearch) > -1) {
                         // if (same size,) then we assume it is our current background...
                         if (fd.nFileSizeLow == gBin.pluginsize) { wcscpy_s(szPlugin, szSearch); }
                     }
                     break;
                }
            }
        }
        while (FindNextFile(hFind, &fd) != 0);
        FindClose(hFind);
    }
    long nSelected = (long) ListFindString(gP.hlistplugin, szPlugin);
    ListSelectPlus(gP.hlistplugin, nSelected);
    if (nSelected < 0) {
        nSelected = 1;
        gBin.pluginsize = FileSize(ListGetText(gP.hlistplugin, nSelected));
    }
    ListSelectPlus(gP.hlistplugin, nSelected);
    skSkinDisable(gP.hlistplugin);
}

void LoadFavorite () {
    if (gP.hfavorite == 0) {
        gP.hfavorite = CreateWindowEx(0, L"LISTBOX", NULL, WS_CHILD, 0, -1000, 0, 0, gP.hwnd, (HMENU) IDC_FAVORITE, gP.instance, NULL);
        if (gP.hfavorite == 0) { return; }
        WCHAR szFile[MAX_PATH] = { 0 };
        PathCombine(szFile, TEMPpath(), $BB64FAV);
        DWORD nSize = FileSize(szFile);
        if (nSize) {
            HANDLE hFile = CreateFile(szFile, GENERIC_READ, 0, NULL, OPEN_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
            if (hFile != INVALID_HANDLE_VALUE) {
                DWORD dwBytesWritten = 0;
                BYTE* buffer = new BYTE[nSize];
                if (ReadFile(hFile, &buffer[0], nSize, &dwBytesWritten, NULL)) {
                    long KK = 0;
                    for (DWORD K = 0; K < nSize; K++) {
                         if (buffer[K] == '\n') {
                             memset(szFile, 0, strSize(szFile));
                             RtlMoveMemory(&szFile[0], &buffer[KK], K - KK);
                             ListAdd(gP.hfavorite, szFile);
                             KK = K + 2;
                         }
                    }
                }
                delete [] buffer;
                CloseHandle(hFile);
            }
        }
        if (ListCount(gP.hfavorite) > 0) { ListSelectPlus(gP.hfavorite, 1); }
        skSkinDisable(gP.hfavorite);
    }
}

void Create_WinLIFT_Controls() {
     HWND hCtrl = 0;
     WCHAR szResource[MAX_PATH] = { 0 };
     long xOffset = -1, yOffset = -1; //-46;

     // Our static border image 
     PathCombine(szResource, EXEresource(), L"Frame.png");
     skStaticImage(gP.hwnd, szResource, xOffset, yOffset, 282, 536, BTN_NOTE3);

     PathCombine(szResource, EXEresource(), L"BTN_List.png");
     hCtrl = skButtonImage(gP.hwnd, szResource, 50 + xOffset, 509 + yOffset, BTN_MENU, 5);
     skCreateToolTip (hCtrl, L"Menu");

     PathCombine(szResource, EXEresource(), L"BTN_Back.png");
     hCtrl = skButtonImage(gP.hwnd, szResource, 216 + xOffset, 508 + yOffset, BTN_BACK, 5);
     skCreateToolTip (hCtrl, L"Not done yet!");

     PathCombine(szResource, EXEresource(), L"BTN_Main.png");
     hCtrl = skButtonImage(gP.hwnd, szResource, 110 + xOffset, 504 + yOffset, BTN_MAIN, 5);
     skCreateToolTip (hCtrl, L"Close");

     hCtrl = CreateWindowEx(0, L"MSCTLS_TRACKBAR32", NULL, WS_CHILD | WS_VISIBLE | WS_TABSTOP | TBS_VERT | TBS_NOTICKS,
                            1 + xOffset, 69 + yOffset, 5, 83, gP.hwnd, (HMENU) IDC_VOLUME, gP.instance, NULL);
     SendMessage(hCtrl, TBM_SETRANGE, 1, MAKELONG(0, 100));
     SendMessage(hCtrl, TBM_SETPOS, 1, 50);
     skSkinChildCtrl(hCtrl, 0);
     skCreateToolTip (hCtrl, L"Sound level");

}

void Create_GDImage_Controls() {
     WCHAR szResource[MAX_PATH] = { 0 };
     HBITMAP hBitmap = 0;
     long x = 0, y = ClientH - (63 + 33), w = 0, h = 0;
     SetRect(&gP.r, x, y, x + ClientW, y + 64);

     PathCombine(szResource, EXEresource(), L"ico_loop.png");
     hBitmap = ZI_CreateBitmapFromFile(szResource, w, h);
     if (hBitmap) { 
         ZD_DrawBitmapToCtrl(gP.gdimage, 254 - w - 4, 449 - h, hBitmap, 0xFFFFFFFF, ID_LOOP, ZS_HIDDEN);
         ZD_SetObjectLocked(ID_LOOP, TRUE); // Lock it.
     }

     PathCombine(szResource, EXEresource(), L"Left_arrow.png");
     hBitmap = ZI_CreateBitmapFromFile(szResource, w, h);
     if (hBitmap) { 
         ZD_DrawBitmapToCtrl(gP.gdimage, -w + 3, (449 - h) / 2, hBitmap, 0xFFFFFFFF, ID_LARROW, ZS_HIDDEN);
         ZD_SetObjectLocked(ID_LARROW, TRUE);
         ZD_SetObjectImageLabel(ID_LARROW, L"prev");
     }

     PathCombine(szResource, EXEresource(), L"right_arrow.png");
     hBitmap = ZI_CreateBitmapFromFile(szResource, w, h);
     if (hBitmap) { 
         ZD_DrawBitmapToCtrl(gP.gdimage, 254 - 3, (449 - h) / 2, hBitmap, 0xFFFFFFFF, ID_RARROW, ZS_HIDDEN);
         ZD_SetObjectLocked(ID_RARROW, TRUE);
         ZD_SetObjectImageLabel(ID_RARROW, L"next");
     }

     PathCombine(szResource, EXEresource(), L"ft00.ttf");
     ZD_DrawTextToCtrlEx(gP.gdimage, TIME(), 7, 20, 150, 50, ZD_ColorARGB(255, RGB(255, 255, 255)),
                         szResource, 51, ID_TIME, ZS_VISIBLE, -1, StringAlignmentNear);
     ZD_SetObjectLocked(ID_TIME, TRUE);

     PathCombine(szResource, EXEresource(), L"ft01.ttf");
     ZD_DrawTextToCtrlEx(gP.gdimage, TODAY(), 12, 70, 150, 46, ZD_ColorARGB(255, RGB(255, 255, 255)),
                         szResource, 14, ID_DATE, ZS_VISIBLE, -1, StringAlignmentNear);
     ZD_SetObjectLocked(ID_DATE, TRUE);

     ZD_DrawTextToCtrlEx(gP.gdimage, L"", 2, 2, 250, 20, ZD_ARGB(255, 255, 255, 255),
                         szResource, 14, ID_TITLE, ZS_VISIBLE, -1, StringAlignmentFar);
     ZD_SetObjectLocked(ID_TITLE, TRUE);

     ZD_DrawTextToCtrlEx(gP.gdimage, L"", 2, 106, 250, 20, ZD_ARGB(255, 255, 255, 255),
                         szResource, 14, ID_ALBUM, ZS_VISIBLE, -1, StringAlignmentCenter);
     ZD_SetObjectLocked(ID_ALBUM, TRUE);

     ZD_DrawTextToCtrlEx(gP.gdimage, L"", 2, 322, 250, 20, ZD_ARGB(255, 255, 255, 255),
                         szResource, 14, ID_ARTIST, ZS_VISIBLE, -1, StringAlignmentCenter);
     ZD_SetObjectLocked(ID_ARTIST, TRUE);

     ZD_DrawTextToCtrlEx(gP.gdimage, L"", 2, 429, 250, 20, ZD_ARGB(255, 255, 255, 255),
                         szResource, 14, ID_GENRE, ZS_VISIBLE, -1, StringAlignmentNear);
     ZD_SetObjectLocked(ID_GENRE, TRUE);

     PathCombine(szResource, EXEresource(), L"pk01.png");
     gP.hpeak = ZI_CreateBitmapFromFile(szResource, w, h);

     w = 200; h = 200;
     PathCombine(szResource, EXEresource(), L"wf01.png");
     hBitmap = ZI_ResizeBitmapFromFile(szResource, w, h);
     if (hBitmap) { 
         ZD_DrawBitmapToCtrl(gP.gdimage, (254 - w) / 2, (449 - h) / 2, hBitmap, 0xFFFFFFFF, ID_WOOFER, ZS_HIDDEN);
         ZD_SetObjectLocked(ID_WOOFER, TRUE);
     }

     PathCombine(szResource, EXEresource(), L"an03.png"); // animated seek tracker
     hBitmap = ZI_CreateBitmapFromFile(szResource, w, h);
     if (hBitmap) { 
         ZD_DrawBitmapToCtrl(gP.gdimage, 0, 0, hBitmap, 0xFFFFFFFF, ID_FLARE, ZS_HIDDEN);
         ZD_SetObjectLocked(ID_FLARE, TRUE);

         ZD_SetObjectFrameCount(ID_FLARE, (BYTE) (w / h));
         ZD_SetObjectFrameToUse(ID_FLARE, 1, FALSE);

     }

     PathCombine(szResource, EXEresource(), L"pon.png"); // Play button
     hBitmap = ZI_CreateBitmapFromFile(szResource, w, h);
     if (hBitmap) {
         ZD_DrawBitmapToCtrl(gP.gdimage, (254 - h) / 2, (449 - h) / 2, hBitmap, 0xFFFFFFFF, ID_PLAYON, ZS_HIDDEN);
         ZD_SetObjectFrameCount(ID_PLAYON, 5);
         ZD_SetObjectFrameToUse(ID_PLAYON, 1, FALSE);
         ZD_SetObjectLocked(ID_PLAYON, TRUE);
     }

     PathCombine(szResource, EXEresource(), L"poff.png"); // Play button
     hBitmap = ZI_CreateBitmapFromFile(szResource, w, h);
     if (hBitmap) {
         ZD_DrawBitmapToCtrl(gP.gdimage, (254 - h) / 2, (449 - h) / 2, hBitmap, 0xFFFFFFFF, ID_PLAYOFF, ZS_HIDDEN);
         ZD_SetObjectFrameCount(ID_PLAYOFF, 5);
         ZD_SetObjectFrameToUse(ID_PLAYOFF, 1, FALSE);
         ZD_SetObjectLocked(ID_PLAYOFF, TRUE);
     }

     // VERY IMPORTANT, we must enable GDImage Events to process mousing and sprite monitoring.
     GDImageEnableEvents(gP.gdimage);
}

void Create_GDImage_Caption() {
     long w = 0, h = 0;
     HBITMAP hBitmap = 0;
     WCHAR szResource[MAX_PATH] = { 0 };

     gP.caption = CreateWindowEx(0, GDImageClassName, L"", WS_CHILD | WS_VISIBLE,
                                 1, 1, ClientW + 26, 43, gP.hwnd, (HMENU) IDC_CAPTION, gP.instance, NULL);

     ZI_UseWinLIFTbackground(gP.caption, TRUE, FALSE); // Use the WinLIFT background.

     // Use this callback to process the GDImage caption control messages.
     ZI_EventMessageEx(gP.caption, (LONG_PTR) CaptionProc, WM_MOUSEMOVE, TRUE);

     // Our caption text.
     PathCombine(szResource, EXEresource(), L"bassbox.ttf");
     //ZD_DrawTextToCtrlEx(gP.caption, L"BASSBOX", 100 + 183, 20, 80, 20, ZD_ColorARGB(255, RGB(195, 195, 195)),
     ZD_DrawTextToCtrlEx(gP.caption, L"BASSBOX", 100, 20, 80, 20, ZD_ColorARGB(255, RGB(195, 195, 195)),
                         szResource, 16, ID_CAPTION, ZS_VISIBLE, 0, StringAlignmentNear);//StringAlignmentCenter);
     ZD_SetObjectLocked(ID_CAPTION, TRUE); // Lock the caption.

     PathCombine(szResource, EXEresource(), L"BTN_Led.png");
     hBitmap = ZI_CreateBitmapFromFile(szResource, w, h);
     if (hBitmap) { 
         ZD_DrawBitmapToCtrl(gP.caption, 225, 6, hBitmap, 0xFFFFFFFF, ID_LED, ZS_VISIBLE);
         ZD_SetObjectFrameCount(ID_LED, (BYTE) (w / h));
         ZD_SetObjectFrameToUse(ID_LED, 1, FALSE);
         ZD_SetObjectLocked(ID_LED, TRUE); // Lock the LED.
     }

     PathCombine(szResource, EXEresource(), L"Twirl24.png");
     hBitmap = ZI_CreateBitmapFromFile(szResource, w, h);
     if (hBitmap) { 
         ZD_DrawBitmapToCtrl(gP.caption, 12, 11, hBitmap, 0xFFFFFFFF, ID_ANICON, ZS_VISIBLE);
         ZD_SetObjectFrameCount(ID_ANICON, (BYTE) (w / h));
         ZD_SetObjectFrameToUse(ID_ANICON, 1, FALSE);
         ZD_SetObjectLocked(ID_ANICON, TRUE); // Lock the ICON.
     }
}

void RemoveQuote(IN WCHAR *szString) {
    if (szString) {
        WCHAR* strip = szString;
        WCHAR c;
        while ((c = *szString++) != 0) {
            if (c != 34) { *strip++ = c; }
        }
        *strip = 0;
    }
}

//void CheckThumb(IN WCHAR* szSrc, IN long Xin, IN long Yin) {
//     WCHAR drive[_MAX_DRIVE], dir[_MAX_DIR], fname[_MAX_FNAME], ext[_MAX_EXT];
//     WCHAR szDst[MAX_PATH] = { 0 };
//     _wsplitpath_s(szSrc, drive, dir, fname, ext);
//     PathCombine(szDst, EXEresource(), L"thb_");
//     wcsncat_s(szDst, fname, wcslen(fname));
//     wcsncat_s(szDst, L".png", 4);
//     if (!FileExist(szDst)) {
//         ZI_SaveBitmapToFile (szDst, ZI_FitImageFrom32Bit (szSrc, Xin, Yin), 0);
//     } else {
//         long w = 0, h = 0;
//         ZI_GetImageSizeFromFile(szDst, w, h);
//         if ((w != Xin) || (h != Yin)) { ZI_SaveBitmapToFile (szDst, ZI_FitImageFrom32Bit (szSrc, Xin, Yin), 0); }
//     }
//}
//
//void Create_Thumbnail() {
//     long nCount = ListCount(gP.hlistback);
//     for (long k = 1; k <= nCount; k++) {
//          CheckThumb(ListGetText(gP.hlistback, k), 52, 91);
//     }
//}

WCHAR* Year() {
    time_t     now = time(0);
    struct tm  tstruct;
    static WCHAR buf[5];
    if (localtime_s(&tstruct, &now) == 0) {;
        wcsftime(buf, strSize(buf), L"%Y", &tstruct);
    }
    return buf;
}

void Create_Menu() {
     HWND hCtrl = 0;
     HBITMAP hBitmap = 0;
     WCHAR szCheck[2];
     WCHAR szResource[MAX_PATH] = { 0 };
     WCHAR szMessage[MAX_PATH] = { 0 };
     RECT r = { 0 }; GetClientRect(gP.gdimage, &r);
     long x = 0, y = 0, w = 220, h = 0, nMax = 8, nHeight = 28, imgW = 0, imgH = 0, UseCheck = 0;
     x = (r.right - w) / 2;
     h = nMax * nHeight;
     y = r.bottom; // - h;
     gP.menudo = 0;

     gP.menu = CreateWindowEx(0, GDImageClassName, L"", WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN,
                                 x, y, w, h, gP.gdimage, (HMENU) IDC_MENU, gP.instance, NULL);
     ZI_SetProperty(gP.menu, ZI_GradientTop, ZD_ARGB(255, 0,132,252));
     ZI_SetProperty(gP.menu, ZI_GradientBottom, ZD_ARGB(255, 0,0,64));

     if (gP.menu) {
         long K, nStep = 0, nID; WCHAR sLabel[128];

         // Equalizer section.
         ZD_DrawRectangleToCtrl (gP.menu, 21, -1, w - (2 + 20), h + 1, ZD_ARGB(255, 0,0,0), 0, ID_MENUBACK1, ZS_HIDDEN, DRAW_FILLED, 0);
         ZD_DrawRectangleToCtrl (gP.menu, 21, -1, w - (2 + 20), h + 2, ZD_ARGB(255, 0,0,0), 1, ID_MENUBACK2, ZS_HIDDEN, DRAW_OUTLINE, 0);
         POINTS pXY[2]; pXY[0].x = (short) 21; pXY[0].y = (short) 85; pXY[1].x = (short) w - 2; pXY[1].y = (short) 85;
         ZD_DrawPolyLineToCtrl (gP.menu, pXY, 2, ZD_ARGB(64, 64,64,64), 1, ID_MENULINE, ZS_HIDDEN, DRAW_OPEN, 0);

         PathCombine(szResource, EXEresource(), L"BTN_TrackV.png");
         x = 25;
         for (K = IDC_EQU_FIRST; K <= IDC_EQU_LAST; K++) {
              nID = K;
              hCtrl = skTrackbar (gP.menu, szResource, x, 10, 18, 150, nID, 100, 0, gBin.equfreq[nID - IDC_EQU_FIRST], ZD_ARGB(255, 0,132,252));
              switch (nID) {
              case IDC_EQU01: skCreateToolTip (hCtrl, L"80 Hz"); break;
              case IDC_EQU02: skCreateToolTip (hCtrl, L"170 Hz"); break;
              case IDC_EQU03: skCreateToolTip (hCtrl, L"310 Hz"); break;
              case IDC_EQU04: skCreateToolTip (hCtrl, L"600 Hz"); break;
              case IDC_EQU05: skCreateToolTip (hCtrl, L"1000 Hz"); break;
              case IDC_EQU06: skCreateToolTip (hCtrl, L"3000 Hz"); break;
              case IDC_EQU07: skCreateToolTip (hCtrl, L"6000 Hz"); break;
              case IDC_EQU08: skCreateToolTip (hCtrl, L"10000 Hz"); break;
              case IDC_EQU09: skCreateToolTip (hCtrl, L"12000 Hz"); break;
              case IDC_EQU10: skCreateToolTip (hCtrl, L"14000 Hz"); break;
              }
              ShowWindow(hCtrl, SW_HIDE);
              x += 19;
         }
         PathCombine(szResource, EXEresource(), L"OffOn.png");
         hBitmap = ZI_CreateBitmapFromFile(szResource, imgW, imgH);
         if (hBitmap) { 
             ZD_DrawBitmapToCtrl(gP.menu, 25, 165, hBitmap, 0xFFFFFFFF, IDC_EQU_OFFON, ZS_HIDDEN);
             ZD_SetObjectFrameCount(IDC_EQU_OFFON, (BYTE) (imgW / imgH));
             ZD_SetObjectFrameToUse(IDC_EQU_OFFON, 1 + gBin.equoffon, FALSE);
             ZD_SetObjectLocked(IDC_EQU_OFFON, TRUE); // Lock the button.
         }
         PathCombine(szResource, EXEresource(), L"ico_redo.png");
         hCtrl = skPushButtonImage(gP.menu, szResource, L"Reset", w - 92, h- 42, 80, nHeight, IDC_EQU_RESET, BS_CENTER | BS_BOTTOM);
         ShowWindow(hCtrl, SW_HIDE);

         // About section.
         PathCombine(szResource, EXEresource(), L"bbicon.png");
         hBitmap = ZI_CreateBitmapFromFile(szResource, imgW, imgH);
         if (hBitmap) { 
             ZD_DrawBitmapToCtrl(gP.menu, 26, 23, hBitmap, 0xFFFFFFFF, ID_BBICON, ZS_HIDDEN);
             ZD_SetObjectLocked(ID_BBICON, TRUE); // Lock the LED.
         }
         PathCombine(szResource, EXEresource(), L"ft01.ttf");
         wcscpy_s(szMessage, L"Version 1.00\n\nAuthor Patrice Terrier\ncopyright (c) ");
         wcsncat_s(szMessage, Year(), 4);
         wcsncat_s(szMessage, L"\nwww.zapsolution.com\n\n\n\n\n\nAudio library\nwww.un4seen.com", 128);
         ZD_DrawTextToCtrlEx(gP.menu, szMessage, 21, 0, w - (2 + 20), h, ZD_ColorARGB(255, RGB(255, 255, 255)),
                             szResource, 14, ID_COPYRIGHT, ZS_VISIBLE, -1, StringAlignmentCenter);
         ZD_SetObjectLocked(ID_COPYRIGHT, TRUE); // Lock the caption.

         // Button menu section.
         // Add the the title BASSBOX on the left edge.
         PathCombine(szResource, EXEresource(), L"bassbox.ttf");
         ZD_DrawTextToCtrlEx(gP.menu, BASSBOX(), 3, h - 100 - 2, 100, 20, ZD_ARGB(255, 0,99,189),
                             szResource, 16, IDC_MENUTITLE, ZS_VISIBLE, 0, ZD_TextVertUp);
         ZD_SetObjectLocked(IDC_MENUTITLE, TRUE); // Lock the title.

         for (K = 0; K < nMax; K++) {
             UseCheck = 0;
             nID = IDC_MENU + K + 1;
             wcscpy_s(szCheck, L" ");
             switch (K) {
             case 0:
                  wcscpy_s(sLabel, L" Add to favorites");
                  PathCombine(szResource, EXEresource(), L"ico_star.png");
                  UseCheck = -1;
                  break;
             case 1:
                  wcscpy_s(sLabel, L" Remove from favorites");
                  PathCombine(szResource, EXEresource(), L"ico_remove.png");
                  UseCheck = -1;
                  break;
             case 2:
                  wcscpy_s(sLabel, L" Play favorites");
                  PathCombine(szResource, EXEresource(), L"ico_heart.png");
                  UseCheck = -1;
                  break;
             case 3:
                  wcscpy_s(sLabel, L" Equalizer settings");
                  PathCombine(szResource, EXEresource(), L"ico_equ.png");
                  if (gBin.equoffon) { wcscpy_s(szCheck, L"a"); }
                  UseCheck = -1;
                  break;
             case 4:
                  wcscpy_s(sLabel, L" Play in loop mode");
                  PathCombine(szResource, EXEresource(), L"ico_loop.png");
                  if (gBin.loopmode) { wcscpy_s(szCheck, L"a"); }
                  UseCheck = -1;
                  break;
             case 5:
                  wcscpy_s(sLabel, L" Enable visual plugins");
                  PathCombine(szResource, EXEresource(), L"ico_plugin.png");
                  if (gBin.useplugin) { wcscpy_s(szCheck, L"a"); }
                  UseCheck = -1;
                  break;
             case 6:
                  wcscpy_s(sLabel, L" Keep player topmost");
                  PathCombine(szResource, EXEresource(), L"ico_param.png");
                  if (gBin.topmost) { wcscpy_s(szCheck, L"a"); }
                  UseCheck = -1;
                  break;
             default:
                  wcscpy_s(sLabel, L" About");
                  PathCombine(szResource, EXEresource(), L"ico_about.png");
                  break;
             }

             hCtrl = skPushButtonImage(gP.menu, szResource, sLabel, 1 + 20, nStep, w - (2 + 20), nHeight, nID, BS_LEFT | BS_BOTTOM);
             ShowWindow(hCtrl, SW_HIDE);
             if (UseCheck) {
                 hCtrl = skPushButtonImage(hCtrl, L"", szCheck, w - (nHeight + 20) , 2, nHeight - 4, nHeight - 4, nID, BS_LEFT | BS_BOTTOM);
                 skSetLabelFont(hCtrl, L"Marlett", 22, skARGB(255, 128,128,128), FontStyleRegular);
                 EnableWindow(hCtrl, FALSE);
             }

             nStep += nHeight;
         }
         // Use this callback to process the GDImage menu control messages.
         ZI_EventMessageEx(gP.menu, (LONG_PTR) MenuProc, WM_COMMAND, TRUE);
         ZI_EventMessageEx(gP.menu, (LONG_PTR) MenuProc, WM_SETCURSOR, TRUE);
         ZI_EventMessageEx(gP.menu, (LONG_PTR) MenuProc, WM_LBUTTONUP, TRUE);
     }

}

////////////////////////////////////////////////////////////////

void KillProcessBBP () {
    WCHAR szFile[MAX_PATH] = { 0 };
    PathCombine(szFile, EXEpath(), $BBP);
    skTerminateProcess(szFile);
}

int WINAPI wWinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPTSTR lpCmdLine, int nCmdShow) {

    UNREFERENCED_PARAMETER(hPrevInstance);

    long nRet = 0, IsInitialized = 0, x = 0, y = 0, w = 0, h = 0, K = 0;
    RECT r = { 0 };
    HWND hWnd = 0, hFound = 0;
    MSG msg = { 0 };
    WNDCLASSEX wcx = { 0 };
    WCHAR szClassName[] = L"ZBB64";
    WCHAR szFile[MAX_PATH] = { 0 };
    BOOL bErrorFlag = 0;
    DWORD dwBytesWritten = 0;
    HANDLE hFile = 0;

    HANDLE hMutex = CreateMutex(NULL, 0, szClassName);
    if (hMutex) {
        if (GetLastError() == ERROR_ALREADY_EXISTS) {
            for (;;) { // Do/Loop
                hFound = FindWindow(szClassName, L""); if (hFound) { break; }
                while (PeekMessage(&msg, NULL, NULL, NULL, PM_REMOVE)) { }
            }
            if (IsIconic(hFound)) { ShowWindow(hFound, SW_RESTORE); }
            SetForegroundWindow(hFound);
            // Use WM_COPYDATA to pass the command line to hFound;
            gCDS.dwData = 0;
            gCDS.cbData = (DWORD) wcslen(lpCmdLine) * 2;
            gCDS.lpData = lpCmdLine;
            SendMessage(hFound, WM_COPYDATA, 0, (LPARAM) &gCDS);
            return 0;
        }
    }

    // Kill any previous instance of BBP.exe if any...
    KillProcessBBP ();

    wcx.cbSize = sizeof(wcx);
    IsInitialized = GetClassInfoEx(hInstance, szClassName, &wcx);
    if (!IsInitialized) {
        wcx.style         = CS_HREDRAW | CS_VREDRAW;
        wcx.lpfnWndProc   = MainProc;
        wcx.cbClsExtra    = 0;
        wcx.cbWndExtra    = 0;
        wcx.hInstance     = hInstance;
        wcx.hIcon         = LoadIcon(hInstance, MAKEINTRESOURCE(IDI_APPLICATION));
        wcx.hCursor       = LoadCursor(NULL, IDC_ARROW);
        wcx.hbrBackground = NULL;
        wcx.lpszMenuName  = NULL;
        wcx.lpszClassName = szClassName;
        wcx.hIconSm       = LoadIcon(wcx.hInstance, MAKEINTRESOURCE(IDI_APPLICATION));
        if (RegisterClassEx(&wcx)) { IsInitialized = -1; }
    }

    if (IsInitialized) {
        DWORD dwExStyle = WS_EX_ACCEPTFILES;
        DWORD dwStyle = WS_POPUP | WS_CLIPSIBLINGS | WS_CLIPCHILDREN; // | WS_VISIBLE
        SetRect(&r, 0, 0, ClientW + 28, ClientH + 40 + 45);
        AdjustWindowRectEx(&r, dwStyle, FALSE, dwExStyle);
        w = r.right - r.left;
        h = r.bottom - r.top;
        x = max((GetSystemMetrics(SM_CXSCREEN) - w) / 2, 0);
        y = max((GetSystemMetrics(SM_CYSCREEN) - h) / 2, 0);
        hWnd = CreateWindowEx(dwExStyle, szClassName, L"", dwStyle, x, y, w, h, NULL, NULL, hInstance, NULL);

        if (hWnd) {

            // Read parameters
            PathCombine(szFile, TEMPpath(), $BB64BIN);
            if (FileExist(szFile)) {
                bErrorFlag = 0; dwBytesWritten = 0;
                hFile = CreateFile(szFile, GENERIC_READ, 0, NULL, OPEN_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
                if (hFile != INVALID_HANDLE_VALUE) { 
                    bErrorFlag = ReadFile(hFile, &gBin, sizeof(gBin), &dwBytesWritten, NULL);
                    CloseHandle(hFile);
                }
            } else {
                gBin.equoffon = 0;
                for (K = 0; K < 10; K++) { gBin.equfreq[K] = (BYTE) 50; }
                gBin.loopmode = 0;
            }

            gP.hwnd = hWnd;
            gP.instance = hInstance;

            RegisterGDImageClass();
            // Detect the active BK??.png and use it to paint the GDImage background.
            DetectBackground();
            DetectPlugin();
            LoadFavorite();
            // Create our playlist container.
            gP.hplaylist = CreateWindowEx(0, L"LISTBOX", L"", WS_CHILD, 0, 0, 0, 0, hWnd, (HMENU)IDC_PLAYLIST, hInstance, NULL);
            // Create our dirlist container.
            gP.hdirlist = CreateWindowEx(0, L"LISTBOX", L"", WS_CHILD, 0, 0, 0, 0, hWnd, (HMENU)IDC_DIRLIST, hInstance, NULL);

            PathCombine(szFile, EXEresource(), ListGetText(gP.hlistback,  ListGetCursel(gP.hlistback)));
            gP.gdimage = CreateWindowEx(0, GDImageClassName, szFile, WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN,
                                        13, 0 + 45, ClientW, ClientH, hWnd, (HMENU)IDC_GDIMAGE, hInstance, NULL);
            if (gP.gdimage) {
                skSkinDisable(gP.gdimage); // Do not skin the main GDImage control.
                PathCombine(szFile, EXEpath(), L"note3.sks");
                if (skInitEngine (szFile, L"")) {
                    skSkinWindow(hWnd, L"Dock|Undock|Minimize|Maximize|Restore|Close");
                    // Add tooltip support to the GDImage control.
//                    ZI_CreateToolTip(gP.gdimage, L"");

                    Create_WinLIFT_Controls();

                    Create_GDImage_Controls();

                    Create_GDImage_Caption();

                    Create_Menu();

//                    Create_Thumbnail();

                    BBP_Handle();

                    if (lpCmdLine) { // Check the command line for audio
                        ProccessCommandline(hWnd, lpCmdLine);
                    }

                }
                // Shall we use topmost mode?
                if (gBin.topmost) { 
                    SetWindowPos(gP.hwnd, HWND_TOPMOST, 0, 0, 0, 0, SWP_NOSIZE | SWP_NOMOVE | SWP_SHOWWINDOW);
                }
            }

            ShowWindow(hWnd, SW_SHOW);

            BBP_Show();

            SetTimer(hWnd, MAIN_TIMER, 0, 0);

            while (GetMessage(&msg, NULL, 0, 0)) {
                TranslateMessage(&msg);
                DispatchMessage(&msg);
            }

            KillTimer(hWnd, MAIN_TIMER);

            if (gP.hpeak) { DeleteObject(gP.hpeak); }
            HMODULE hGenie = AudioGenie(); if (hGenie) { FreeLibrary(hGenie); }

            nRet = (long)msg.wParam;
        }
    }
    if (hMutex) { CloseHandle(hMutex); }

    // Save parameters
    bErrorFlag = 0; dwBytesWritten = 0;
    PathCombine(szFile, TEMPpath(), $BB64BIN);
    hFile = CreateFile(szFile, GENERIC_WRITE, 0, NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
    if (hFile != INVALID_HANDLE_VALUE) { 
        bErrorFlag = WriteFile(hFile, &gBin, sizeof(gBin), &dwBytesWritten, NULL);
        CloseHandle(hFile);
    }

    return nRet;
}
